Package: R2HTML
Version: 2.3.4
Date: 2024-06-16
Title: HTML Exportation for R Objects
Authors@R: c(person(given="Eric", family="Lecoutre", email="eric.lecoutre@gmail.com",
                    role=c("aut")),
             person(given="Milan", family="Bouchet-Valat", email="nalimilan@club.fr",
                    role=c("cre", "ctb")),
             person(given="Thomas", family="Friedrichsmeier", email="tfry@users.sf.net",
                    role=c("ctb")))
Depends: R (>= 2.0)
Imports: stats, graphics, utils, grDevices, methods
Suggests: boot, splines, survival, cluster, nlme, rpart, nnet
Description: Includes HTML function and methods to write in an HTML
        file. Thus, making HTML reports is easy. Includes a function
        that allows redirection on the fly, which appears to be very
        useful for teaching purpose, as the student can keep a copy of
        the produced output to keep all that he did during the course.
        Package comes with a vignette describing how to write HTML
        reports for statistical analysis. Finally, a driver for 'Sweave'
        allows to parse HTML flat files containing R code and to
        automatically write the corresponding outputs (tables and
        graphs).
License: GPL (>= 2)
URL: https://github.com/nalimilan/R2HTML
NeedsCompilation: no
Packaged: 2024-06-16 13:48:36 UTC; milan
Author: Eric Lecoutre [aut],
  Milan Bouchet-Valat [cre, ctb],
  Thomas Friedrichsmeier [ctb]
Maintainer: Milan Bouchet-Valat <nalimilan@club.fr>
Repository: CRAN
Date/Publication: 2024-06-16 17:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 17:22:06 UTC; unix
