import(graphics,utils)
importFrom("grDevices", "bitmap", "dev.off", "dev.print", "grey",
           "jpeg", "png")
importFrom("methods", "is")
importFrom("stats", "as.ts", "asOneSidedFormula", "coef", "end",
           "formula", "frequency", "median", "naprint", "pchisq", "pf",
           "quantile", "start", "symnum", "time", "tsp", "var")

export(HTML,
       HTMLSetFile,
       HTMLGetFile,
       HTML.title,
       HTMLstem,
       HTMLbr,
       HTMLhr,
       HTMLli,
       HTMLplot,
       HTMLInsertGraph,
       HTMLCSS,
       HTMLChangeCSS,
       HTML.cormat,
       HTMLStart,
       HTMLInitFile,
       HTMLEndFile,
       HTMLStop,
       RweaveHTML,
       RweaveHTMLSetup,
       RweaveHTMLRuncode,
       RweaveHTMLWritedoc,
       RweaveHTMLFinish,
       RweaveHTMLOptions,
       SweaveSyntaxHTML,
       as.title,
       as.latex,
       HTMLgrid_references,
       HTMLgrid,
       HTMLgrid_inline,
       HTMLgrid_summary,
       HTML2clip
       )

S3method(HTML, default)
S3method(HTML, atomic)
S3method(HTML, complex)
S3method(HTML, numeric)
S3method(HTML, integer)
S3method(HTML, logical)
S3method(HTML, character)
S3method(HTML, call)
S3method(HTML, "function")
S3method(HTML, environment)
S3method(HTML, formula)
S3method(HTML, array)
S3method(HTML, by)
S3method(HTML, family)
S3method(HTML, terms)
S3method(HTML, factor)
S3method(HTML, density)
S3method(HTML, infl)
S3method(HTML, lm)
S3method(HTML, lm.null)
S3method(HTML, ftable)
S3method(HTML, POSIXlt)
S3method(HTML, POSIXct)
S3method(HTML, octmode)
S3method(HTML, rle)
S3method(HTML, logLik)
S3method(HTML, xtabs)
S3method(HTML, summary.lm)
S3method(HTML, coefmat)
S3method(HTML, table)
S3method(HTML, listof)
S3method(HTML, ts)
S3method(HTML, list)
S3method(HTML, pairlist)
S3method(HTML, data.frame)
S3method(HTML, matrix)
S3method(HTML, structure)
S3method(HTML, connection)
S3method(HTML, socket)
S3method(HTML, htest)
S3method(HTML, aov)
S3method(HTML, anova)
S3method(HTML, glm)
S3method(HTML, tables.aov)
S3method(HTML, mtable)
S3method(HTML, integrate)
S3method(HTML, summary.lm.null)
S3method(HTML, summary.glm)
S3method(HTML, hsearch)
S3method(HTML, aovlist)
S3method(HTML, SavedPlots)
S3method(HTML, ordered)
S3method(HTML, difftime)
S3method(HTML, dummy.coef)
S3method(HTML, dummy.coef.list)
S3method(HTML, glm.null)
S3method(HTML, MethodsFunction)
S3method(HTML, libraryIQR)
S3method(HTML, summary.aov)
S3method(HTML, summary.aovlist)
S3method(HTML, summary.glm.null)
S3method(HTML, summary.manova)
S3method(HTML, summary.table)
S3method(HTML, TukeyHSD)
S3method(HTML, simple.list)
S3method(HTML, noquote)
S3method(HTML, ar)
S3method(HTML, Arima)
S3method(HTML, arima0)
S3method(HTML, HoltWinters)
S3method(HTML, stl)
S3method(HTML, StructTS)
S3method(HTML, tskernel)
S3method(HTML, pairwise.htest)
S3method(HTML, power.htest)
S3method(HTML, boot)
S3method(HTML, simplex)
S3method(HTML, saddle.distn)
S3method(HTML, bootci)
S3method(HTML, dist)
S3method(HTML, factanal)
S3method(HTML, loadings)
S3method(HTML, hclust)
S3method(HTML, prcomp)
S3method(HTML, princomp)
S3method(HTML, summary.prcomp)
S3method(HTML, summary.princomp)
S3method(HTML, medpolish)
S3method(HTML, tukeyline)
S3method(HTML, tukeysmooth)
#S3method(HTML, grob)
S3method(HTML, unit)
S3method(HTML, viewport)
S3method(HTML, shingle)
S3method(HTML, shingleLevel)
S3method(HTML, abbrev)
S3method(HTML, Anova)
S3method(HTML, anova.loglm)
S3method(HTML, correspondence)
S3method(HTML, fitdistr)
S3method(HTML, fractions)
S3method(HTML, gamma.shape)
S3method(HTML, glm.dose)
S3method(HTML, lda)
S3method(HTML, loglm)
S3method(HTML, mca)
S3method(HTML, polr)
S3method(HTML, qda)
S3method(HTML, ridgelm)
S3method(HTML, rlm)
S3method(HTML, rms.curv)
S3method(HTML, summary.loglm)
S3method(HTML, summary.negbin)
S3method(HTML, summary.polr)
S3method(HTML, summary.rlm)
S3method(HTML, multinom)
S3method(HTML, nnet)
S3method(HTML, summary.multinom)
S3method(HTML, summary.nnet)
S3method(HTML, agnes)
S3method(HTML, clara)
S3method(HTML, diana)
S3method(HTML, dissimilarity)
S3method(HTML, ellipsoid)
S3method(HTML, fanny)
S3method(HTML, mona)
S3method(HTML, pam)
S3method(HTML, summary.agnes)
S3method(HTML, summary.clara)
S3method(HTML, summary.diana)
S3method(HTML, summary.fanny)
S3method(HTML, summary.mona)
S3method(HTML, summary.pam)
S3method(HTML, gam)
S3method(HTML, summary.gam)
S3method(HTML, rpart)
S3method(HTML, loess)
S3method(HTML, ppr)
S3method(HTML, smooth.spline)
S3method(HTML, summary.loess)
S3method(HTML, summary.ppr)
S3method(HTML, bSpline)
S3method(HTML, polySpline)
S3method(HTML, ppolySpline)
S3method(HTML, lqs)
S3method(HTML, nls)
S3method(HTML, summary.nls)
S3method(HTML, ecdf)
S3method(HTML, stepfun)
S3method(HTML, cox.zph)
S3method(HTML, coxph.null)
S3method(HTML, xtable)
S3method(HTML, latex)
S3method(HTML, coefmat)
S3method(HTML, reStruct)
S3method(HTML, summary.lme)   
S3method(HTML, summary.pdDiag)
S3method(HTML, title)
S3method(HTML, cormat)
